@@ -33,7 +33,7 @@ class DatabaseTrace(newrelic.api.time_trace.TimeTrace):
 
     def __init__(self, transaction, sql, dbapi2_module=None,
                  connect_params=None, cursor_params=None,
-                 execute_params=None):
+                 sql_parameters=None, execute_params=None):
 
         super(DatabaseTrace, self).__init__(transaction)
 
@@ -46,6 +46,7 @@ class DatabaseTrace(newrelic.api.time_trace.TimeTrace):
 
         self.connect_params = connect_params
         self.cursor_params = cursor_params
+        self.sql_parameters = sql_parameters
         self.execute_params = execute_params
 
     def dump(self, file):
@@ -56,6 +57,7 @@ class DatabaseTrace(newrelic.api.time_trace.TimeTrace):
 
         connect_params = None
         cursor_params = None
+        sql_parameters = None
         execute_params = None
 
         settings = self.transaction.settings
@@ -80,11 +82,13 @@ class DatabaseTrace(newrelic.api.time_trace.TimeTrace):
             # cause further problems.
 
             if (exc is None and transaction_tracer.explain_enabled and
-                    self.duration >= transaction_tracer.explain_threshold):
+                    self.duration >= transaction_tracer.explain_threshold and
+                    self.connect_params is not None):
                 if (self.transaction._explain_plan_count <
                        agent_limits.sql_explain_plans):
                     connect_params = self.connect_params
                     cursor_params = self.cursor_params
+                    sql_parameters = self.sql_parameters
                     execute_params = self.execute_params
                     self.transaction._explain_plan_count += 1
 
@@ -92,6 +96,7 @@ class DatabaseTrace(newrelic.api.time_trace.TimeTrace):
 
         self.connect_params = connect_params
         self.cursor_params = cursor_params
+        self.sql_parameters = sql_parameters
         self.execute_params = execute_params
 
     def create_node(self):
@@ -101,6 +106,7 @@ class DatabaseTrace(newrelic.api.time_trace.TimeTrace):
                 exclusive=self.exclusive, stack_trace=self.stack_trace,
                 sql_format=self.sql_format, connect_params=self.connect_params,
                 cursor_params=self.cursor_params,
+                sql_parameters=self.sql_parameters,
                 execute_params=self.execute_params)
 
     def terminal_node(self):
